GameWorld = class("GameWorld")
kGroundUserData = "Ground"
kGroundSensorData = "GroundSensor"
kObstacleUserData = "Obstacle"
groundSensorRadius = 20

kPlayerCollisionCategory = 1
kGroundCollisionCategory = 2
kObstacleCollisionCategory = 3

kPhysicDt = 1/30

function GameWorld:initialize(level, players)
	super.initialize()

	love.physics.setMeter(64) --the height of a meter our worlds will be 64px

	self.world = love.physics.newWorld(0, 9.81*64, true) --create a world with gravity
	self.world:setCallbacks(function(a,b,coll) self:beginContact(a,b,coll) end,
												 function(a,b,coll) self:endContact(a,b,coll) end,
												 function(a,b,coll) self:preSolve(a,b,coll) end,
												 function(a,b,coll) self:postSolve(a,b,coll) end)
	self.objects = {} -- table to hold all our physical objects	

	self.level = level
	self.players = players
	self.objects = {} -- table to hold all our physical objects

	-- create level content as physical objects
	self.objects.ground = {}
	
	for k, part in pairs(level.parts) do
		self:createPart(part, kGroundUserData, kGroundCollisionCategory)
		local sensorPart = CopyArray(part)
		for i = 6,8,2 do
			--~ print("Sensorpart ", sensorPart[i])
			sensorPart[i] = sensorPart[i] - groundSensorRadius
		end
		local groundSensor = self:createPart(sensorPart, kGroundSensorData, kGroundCollisionCategory)
		groundSensor.fixture:setSensor(true)
	end
	
	for k, part in pairs(level.obstacles) do
		self:createPart(part, kObstacleUserData, kObstacleCollisionCategory)
	end
	
	-- create players as physical objects
	local i = 1
	for k, player in pairs(self.players) do
		local playerObject = {}
		playerObject.body = love.physics.newBody(self.world, player.x, player.y, "dynamic")
		playerObject.shape = love.physics.newCircleShape(20)
		playerObject.fixture = love.physics.newFixture(playerObject.body, playerObject.shape,1)
		playerObject.fixture:setUserData(k)
		playerObject.fixture:setCategory(kPlayerCollisionCategory)
		playerObject.fixture:setMask(kPlayerCollisionCategory)
		player.physicsObject = playerObject
		player.setWorld = self.world
		i = i + 1
	end
end

function GameWorld:createPart(part, name, collisionCategory)
	local object = {}
	object.body = love.physics.newBody(self.world, 0, 0) --remember, the shape (the rectangle we create next) anchors to the body from its center, so we have to move it to (650/2, 650-50/2)
	object.shape = love.physics.newPolygonShape(unpack(part)) --make a rectangle with a width of 650 and a height of 50
	object.fixture = love.physics.newFixture(object.body, object.shape) --attach shape to body
	object.fixture:setUserData(name)
	object.fixture:setCategory(collisionCategory)
	object.fixture:setMask(kObstacleCollisionCategory, kGroundCollisionCategory)

	table.insert(self.objects, object)

	return object
end

function GameWorld:update(dt)
	self.world:update(dt)
end

function GameWorld:beginContact(a, b, coll)
	local player = self.players[a:getUserData()] or self.players[b:getUserData()]
	local isGround = a:getUserData()==kGroundSensorData or b:getUserData()==kGroundSensorData
	local isObstacle = a:getUserData()==kObstacleUserData or b:getUserData()==kObstacleUserData
	if (isGround and player) then
		--~ print("GROUND")
		player.groundConnectionCounter = player.groundConnectionCounter + 1
		self:playerCollided(kGroundCollisionCategory, player)
	end
	if (isObstacle and player) then
		--~ print("OBSTACLE")
		player.groundConnectionCounter = player.groundConnectionCounter + 1
		self:playerCollided(kObstacleCollisionCategory, player)
	end
end

function GameWorld:endContact(a, b, coll)
  local player = self.players[a:getUserData()] or self.players[b:getUserData()]
	local isGround = a:getUserData()==kGroundSensorData or b:getUserData()==kGroundSensorData
	local isObstacle = a:getUserData()==kObstacleUserData or b:getUserData()==kObstacleUserData
	if ((isGround or isObstacle) and player) then
		player.groundConnectionCounter = player.groundConnectionCounter - 1
	end
end

function GameWorld:preSolve(a, b, coll)
   
end

function GameWorld:postSolve(a, b, coll)
   
end

-- received values here are:
-- kGroundCollisionCategory on collision with ground
-- kObstacleCollisionCategory on collision with obstacles
function GameWorld:playerCollided(target, player)
	--~ local vx,vy = player:getVelocity()
	--~ local vel = len2(vx,vy)
	if player.groundConnectionCounter == 1 then
		--print("hit")
		TEsound.play("media/sounds/hit.ogg", "hit", 0.2)
	end
end
