-- Class
Generator = class("Generator")

function Generator:initialize(partLength, partCount, baseY, minHeight, maxHeight, 
	holePercentage, obstaclePercentage, obstacleUpPercentage, obstacleUpHeight, obstacleSize)
	
	super.initialize(self)
	
	self.partLength = partLength
	self.partCount = partCount
	self.minHeight = minHeight
	self.maxHeight = maxHeight
	self.maxLength = maxLength
	self.baseY = baseY
	self.holePercentage = holePercentage
	self.obstaclePercentage = obstaclePercentage
	self.obstacleUpPercentage = obstacleUpPercentage
	self.obstacleUpHeight = obstacleUpHeight
	self.minObstacleSize = obstacleSize
	self.maxObstacleSize = obstacleSize
	self.minPlatoHeightDelta = 50
	self.maxPlatoHeightDelta = 100
	self.platoChangePercentage = 0.3
	self.itemsStartAtPart = 5
	
	self.itemPercentage = 0.1
end

function Generator:generateAndAddToLevel(level)
	local lasth = nil
	local lastWasHole = true
	local lastWasPlatoChange = false
	local lastWasObstacle = false
	
	local platoH = 0
	
	local d = self.partLength
	 
	for i = 0,self.partCount-1 do
		local isPlatoChange = false
		-- adjust plato height
		if not lastWasObstacle and math.random() < self.platoChangePercentage then
			platoH = platoH + (math.random(1,2) == 1 and -1 or 1) *
				(self.minPlatoHeightDelta + math.random() * (self.maxPlatoHeightDelta - self.minPlatoHeightDelta))
			platoH = math.max(0, math.min(self.baseY / 3, platoH))
			isPlatoChange = true
		end
	
		local x = d * i
		local y = self.baseY
		local h = math.random(self.minHeight, self.maxHeight) + platoH
		
		-- ground
		if not lasth or isPlatoChange then lasth = h end
		
		local isHole = i > 5 and not lastWasHole and math.random() < self.holePercentage
		
		if not isHole then 
			level:addPart({x,y, x+d,y, x+d,y-h, x,y-lasth})
		end
		
		local maxH = math.max(lashh or h, h)
		local medH = (lasth + h) / 2
		
		lasth = h
		
		local hasObstacle = false
		
		-- obstacles
		if not isPlatoChange and not isHole and math.random() < self.obstaclePercentage then
			local isUp = math.random() < self.obstacleUpPercentage
			
			local xx = x + d / 2
			local yy = self.baseY - maxH - self.obstacleBase
			if isUp then yy = yy - self.obstacleUpHeight end
			
			local s = self.minObstacleSize + math.random() * (self.maxObstacleSize - self.minObstacleSize)
			local sd = s / 2
			
			level:addObstacle({xx-sd,yy-sd, xx+sd,yy-sd, xx+sd,yy+sd, xx-sd,yy+sd})
			
			hasObstacle = true
		end
		
		-- items
		if i > self.itemsStartAtPart and not hasObstacle and not isHole and math.random() < self.itemPercentage then
			local r = 20
			local xx = x + d / 2
			local yy = self.baseY - medH - r
			
			local itemType = math.random(1,25)
			if itemType < 9 then
				itemType = 1
			elseif itemType < 19 then
				itemType = 2
			else
				itemType = 3
			end
			level:addItem(Item:new(xx, yy, r, itemType))
		end
		
		lastWasHole = isHole
		lastWasObstacle = hasObstacle
		lastWasPlatoChange = isPlatoChange
	end
end
