jumpForceY = -150
jumpForceX = 20

-- Class
Player = class("Player")

function Player:initialize(x,y, id)
	super.initialize(self)
	self.physicsObject = nil
	self.world = nil
	self.x = x
	self.y = y
	self.startY = y
	self.deaths = 0
	self.id = id
	self.animTime = math.random(0, 100)/100
	self.animDir = 1
	
	self.forwardForceFactor = 1
	self.minimalVelocity = 300
	self.velocityIncrease = 5
	
	self.jumpPossible = true
	self.playerKeyPressed = false
	self.playerKey = nil
	
	if self.id == 1 then
		self.colour = {255, 128, 0, 255}
	elseif self.id == 2 then
		self.colour = {255, 0, 128, 255}
	elseif self.id == 3 then
		self.colour = {255, 128, 128, 255}
	elseif self.id == 4 then
		self.colour = {0, 128, 255 , 255}
	end

	self.groundConnectionCounter = 0

	self.part1 = love.graphics.newImage("media/images/part1.png");
	self.ps = love.graphics.newParticleSystem( self.part1, 100 )
	self.ps:setEmissionRate( 100 )
	self.ps:setSpeed(0, 0)
	self.ps:setGravity(0)
	self.ps:setSizes(2, 1)
	self.ps:setColors(self.colour[1], self.colour[2], self.colour[3], self.colour[4], 194, 30, 18, 0)
	self.ps:setPosition(x-50, y-5)
	self.ps:setParticleLife(0.5)
	self.ps:setDirection(0)
	self.ps:setSpread(360)
	self.ps:setRadialAcceleration(-2000)
	self.ps:setTangentialAcceleration(1000)
	self.ps:start()
end

function Player:update(dt)
	self.minimalVelocity = self.minimalVelocity + self.velocityIncrease * dt
	
	if (self.physicsObject) then
		local px = self.physicsObject.body:getX()
		local py = self.physicsObject.body:getY()
		
		self.x = px
		self.y = py
		local velx, vely = self:getVelocity()
		if (velx < self.minimalVelocity*self.forwardForceFactor) then
			velx = self.minimalVelocity
			self.physicsObject.body:setLinearVelocity(velx, vely)
		end
	end
	
	self:jump(dt)

	-- death respawn
	if self.y > kYDeathTreshold then

		TEsound.play("media/sounds/die.ogg", "die")
--[[		self.deaths = self.deaths + 1
		self.y = self.startY
		self.x = self.x + kXDeathDelta
		
		if (self.physicsObject) then
			self.physicsObject.body:setPosition(self.x, self.y)
		end
		
		self.forwardForceFactor = 1
]]--
		-- kill player
		for k,player in pairs(gGame.players) do
			if gGame.players[k] == self then
				gGame.players[k] = nil
				print("remove player " .. self.id)
				return
			end
		end
	end

	self.animTime = self.animTime + self.animDir * dt
	if self.animTime > 1.0 then 
		self.animDir = -1 
	elseif self.animTime < 0.0 then
		self.animDir = 1
	end
	
	-- allow jump again
	if not self.jumpPossible and not self.playerKeyPressed then
		self.jumpPossible = true
	end
	
	-- game end check
	if self.x > gGame.levelEndX then
		gGame.winner = self.playerKey
	end

	self.ps:setSpeed(self:getVelocity())
	self.ps:update(dt)
end

function Player:jump(dt)
	if (self.physicsObject and self.groundConnectionCounter > 0 and self.playerKeyPressed and self.jumpPossible) then
		self.physicsObject.body:applyLinearImpulse( jumpForceX, jumpForceY )
		self.jumpPossible = false
		TEsound.play("media/sounds/jump.ogg", "jump")
	end
end

function Player:draw(drawcircle)
	local r = self.physicsObject and self.physicsObject.shape:getRadius() or 10
	
--	love.graphics.setColor(self.colour)
--	love.graphics.setLineWidth(1./_globalScale_ * _screenWidth_/200)
--	love.graphics.circle("line", self.x, self.y, r + r/4 * self.animTime)

	love.graphics.setColorMode("modulate")
	love.graphics.setBlendMode("additive")
	love.graphics.draw(self.ps, self.x-50, self.y-5)

	-- draw slime and rest with one color
	love.graphics.setBlendMode("alpha")
	love.graphics.setColor(self.colour)
	self:drawLate()
	
	if drawcircle == true then
		love.graphics.setLineWidth(1.)
		love.graphics.circle("line", self.x, self.y, r + r/4 * self.animTime)
	end
	
	if self.playerKey then
		love.graphics.setFont(font)
		local name = self.playerKey
		if name == " " then name = "space" end
		love.graphics.print(name, self.x + r + 10, self.y - r + 10)
	end
end

function Player:getSpeed()
	if self.physicsObject then
		local vx,vy = self.physicsObject.body:getLinearVelocity()
		return vx
	else
		return 0
	end
end

function Player:getVelocity()
	if self.physicsObject then
		return self.physicsObject.body:getLinearVelocity()
	else
		return 0,0
	end
end

function Player:boost(faster)
	if faster then
		TEsound.play("media/sounds/speed-up.ogg", "speed-up")
		self.forwardForceFactor = self.forwardForceFactor * 1.1
	else
		TEsound.play("media/sounds/speed-down.ogg", "speed-down")
		self.forwardForceFactor = self.forwardForceFactor * 0.9
	end
end

function Player:drawLate()
	local r = self.physicsObject and self.physicsObject.shape:getRadius() or 10
	local fac = 0.5 + 0.15 * self.animTime

--	love.graphics.setColor(self.colour)
	love.graphics.setLineWidth(1./_globalScale_ * _screenWidth_/600)
	love.graphics.circle("fill", self.x, self.y, r/4 * fac) --nose

	
	love.graphics.circle("line", self.x - r/4 - r/2 * fac, self.y - r/2 * fac, (r + r/4 * self.animTime)/8) --eyes
	love.graphics.circle("line", self.x + r/4 + r/2 * fac, self.y - r/2 * fac, (r + r/4 * self.animTime)/8)
	
	love.graphics.arc( "line", self.x, self.y + r/3, 2*r/3 * fac, 0.00, 3.14) --mouth
end
