local function FindSmallestPO2(num)
	return 2 ^ math.ceil(math.log(num)/math.log(2))
end

local function ScaleToPO2(xsize, ysize)
	if love.graphics.isSupported("npot") then return xsize, ysize end
	return FindSmallestPO2(xsize), FindSmallestPO2(ysize)
end

function CreateFlipEffect(xsize, ysize) 
	if not love.graphics.newPixelEffect
	or not love.graphics.isSupported
	or not love.graphics.isSupported("pixeleffect")
	or not love.graphics.isSupported("canvas") then
		return
	end
	
	local po2xsize, po2ysize = ScaleToPO2(xsize, ysize) -- scales x and y to next power of 2 if npot is false
		
	
	local xres, yres = love.graphics.getWidth(), love.graphics.getHeight()
	local po2xres, po2yres = ScaleToPO2(xres, yres)

	
	local quad = love.graphics.newQuad(0, 0, xsize, ysize, po2xsize, po2ysize)
	local scenequad = love.graphics.newQuad(0, 0, xres, yres, po2xres, po2yres)

	local state = 0
	local timePassed = 0

	local flip = {}
	
	local canvas = {
		scene = love.graphics.newCanvas(po2xres, po2yres),
	}
	
	local shaders = {
		zoom = love.graphics.newPixelEffect[[
			extern number zoom = 1.0;
			
			vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
			{				
				return Texel(texture, zoom * texture_coords);
			}
		]],
		flip = love.graphics.newPixelEffect[[
			vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
			{
				return Texel(texture, vec2(texture_coords.x, 1.0 - texture_coords.y));
			}
		]]
	}

	function flip:setZoom(zoom)
		shaders.zoom:send("zoom", 4*zoom)
	end
	
	
	flip:setZoom(1.0)
	
	-- call right before drawing the stuff you want bloomed
	function flip:predraw()
		if state == 0 then
			return
		end
		for k,v in pairs(canvas) do
			v:clear()
		end
		love.graphics.setCanvas(canvas.scene)
	end

	function flip:update(dt)
		local ret = true
		if state > 0 then
			timePassed = timePassed + dt
			if state == 1 then
				flip:setZoom(1.0 - timePassed)
				if timePassed > 0.25 then
					timePassed = 0.0
					state = 2
				end
			elseif state == 2 then
				if timePassed > 3.0 then
					timePassed = 0.0
					state = 3
				end
			elseif state == 3 then
				flip:setZoom(timePassed)
				if timePassed > 0.25 then
					timePassed = 0.0
					state = 0
					ret = false
				end
			end
		end
		return ret
	end

	function flip:trigger()
		if state == 0 then
			state = 1
		end
	end
	
	-- call after drawing the stuff you want bloomed
	function flip:postdraw()	
		if state == 0 then
			return
		end
		love.graphics.setCanvas()	
		love.graphics.setColor(255, 255, 255)
		love.graphics.setBlendMode("alpha")
		
		
		-- render final scene combined with bloom canvas
		if state == 1 or state == 3 then
			love.graphics.setPixelEffect(shaders.zoom)
		elseif state == 2 then
			love.graphics.setPixelEffect(shaders.flip)
		end
		love.graphics.drawq(canvas.scene, scenequad, 0, 0)
		
		love.graphics.setPixelEffect()
		
		love.graphics.setBlendMode("alpha")
	end
	
	return flip
end
