-- apt-get install liblua5.1-socket2
local socket = require("socket") -- luasocket
--~ print("lib.genserv.lua: luasocket",socket)
-- luasocket docs : http://w3.impa.br/~diego/software/luasocket/introduction.html
-- luasocket docs : http://w3.impa.br/~diego/software/luasocket/socket.html
-- luasocket docs : http://w3.impa.br/~diego/software/luasocket/tcp.html
-- nc -lk "*" 2323

-- returns time since some fixed point in milliseconds
function RobGetTicks () return os.clock()*1000 end

kMsgTypes = {}
kMsgTypes.kMsgType_Undefined	= 0
kMsgTypes.kMsgType_NormalUDP	= 10
kMsgTypes.kMsgType_ReliableUDP	= 11
kMsgTypes.kMsgType_Ack			= 12
kMsgTypes.kMsgType_JoinRoom		= 13
kMsgTypes.kMsgType_LeaveRoom	= 14
kMsgTypes.kMsgType_Hello		= 15
kMsgTypes.kMsgType_Welcome		= 16
kMsgTypes.kMsgType_Quit			= 17
kMsgTypes.kMsgType_Ping			= 18

local kMsgTypeTxt = {}
for k,v in pairs(kMsgTypes) do _G[k] = v kMsgTypeTxt[v] = k end

--~ local kReadTimeout_Head		= 0 -- 100/1000
--~ local kReadTimeout_Data		= nil -- infinite

kKnownReliableUDPAckCheckInterval = 1000

--cUDPServCon = CreateClass()
cUDPServCon = class("cUDPServCon")



-- ***** ***** ***** ***** ***** construct

function cUDPServCon:initialize (address,port,timeout) -- timeout in 
	print("cUDPServCon",address,port)
	self.socket = socket.udp() assert(self.socket)
	assert(self.socket:setpeername(address,port)) -- becomes client socket : send, receive, getsockname, getpeername, settimeout, close. 
	self.socket:settimeout(timeout or (1/1000))
	--~ self.socket:settimeout(kReadTimeout_Head)
	self.mAckHandler = {}
	self.mTasks = {} -- executed at step
	self.mRoomIdByName = {}
	self.mRoomNameById = {}
	self.mOwnID = nil
	self.mLastAckId = 0
	self.miNextKnownReliableUDPAckCheck = 0
	self.mKnownReliableUDPAck = {}
end

function cUDPServCon:Close ()
	self:SendQuit()
	if (self.socket) then
		self.socket:close()
		self.socket = nil
	end
end


-- ***** ***** ***** ***** ***** utils

local floor = math.floor
-- print("fourbyte_w",i,a,b,c)
--~ local function fourbyte_w (i) return string.char(floor(i/(256*256)) % 256,floor(i/256) % 256,floor(i) % 256) end
local function byte1_w (i) return string.char(i) end
local function byte4_w (i) local a,b,c,d = floor(i) % 256,floor(i/256) % 256,floor(i/(256*256)) % 256,floor(i/(256*256*256)) % 256 return string.char(a,b,c,d) end
local function byte4_w_hton (i) local d,c,b,a = floor(i) % 256,floor(i/256) % 256,floor(i/(256*256)) % 256,floor(i/(256*256*256)) % 256 return string.char(a,b,c,d) end
local function data_rest (s,off) off = off or 0 return string.sub(s,off+1) end
local function byte1_r (s,off) off = off or 0 return string.byte(s,off+1,off+1) end
local function byte4_r (s,off) 
	off = off or 0
	local a,b,c,d = string.byte(s,off+1,off+4)
	return a+256*(b+256*(c+256*d))
end
local function byte4_r_ntoh (s,off) 
	off = off or 0
	local a,b,c,d = string.byte(s,off+1,off+4)
	return d+256*(c+256*(b+256*a))
end

local function hex (x) return (type(x) == "number") and string.format("%#x",math.floor(x)) end 

function cUDPServCon:Sleep (msecs) socket.sleep(msecs/1000) end

-- ***** ***** ***** ***** ***** handlers

function cUDPServCon:ExecuteAckHandler (iAckId,pPacket)
	local h = self.mAckHandler[iAckId]
	--~ print("ExecuteAckHandler",iAckId,h)
	if (h) then 
		self.mAckHandler[iAckId] = nil
		h(pPacket)
	end
end
function cUDPServCon:AddAckHandler (iAckId,h) self.mAckHandler[iAckId] = h end
function cUDPServCon:AddTask (task) self.mTasks[task] = true end
function cUDPServCon:DelTask (task) self.mTasks[task] = nil end

-- ***** ***** ***** ***** ***** messages

-- send normal udp message, not reliable
function cUDPServCon:Send			(iTo,pData) 		self:SendRaw(byte1_w(kMsgType_NormalUDP)..byte4_w_hton(iTo)..pData) end

-- don't use directly, see StartSendReliable
function cUDPServCon:SendReliable	(iAckId,iTo,iInterval,pData)
	self:SendRaw(byte1_w(kMsgType_ReliableUDP)..byte4_w_hton(iAckId)..byte4_w_hton(iTo)..byte4_w_hton(iInterval)..pData)
end

function cUDPServCon:SendAck		(iAckId)			self:SendRaw(byte1_w(kMsgType_Ack)..byte4_w_hton(iAckId))	end

-- don't use directly, see StartJoinRoom
function cUDPServCon:SendJoinRoom	(iAckId,sRoomName)	self:SendRaw(byte1_w(kMsgType_JoinRoom)..byte4_w_hton(iAckId)..sRoomName) end

-- don't use directly, see StartLeaveRoom
function cUDPServCon:SendLeaveRoom	(iAckId,iRoomId) 	self:SendRaw(byte1_w(kMsgType_LeaveRoom)..byte4_w_hton(iAckId)..byte4_w_hton(iRoomId)) end

function cUDPServCon:SendHello		() self:SendRaw(byte1_w(kMsgType_Hello)) end
function cUDPServCon:SendQuit		() self:SendRaw(byte1_w(kMsgType_Quit)) end
function cUDPServCon:SendPing		() self:SendRaw(byte1_w(kMsgType_Ping)) end

function cUDPServCon:SendRaw		(pPacket) self.socket:send(pPacket) end

-- ***** ***** ***** ***** ***** high level

function cUDPServCon:GenerateAckId		()
	self.mLastAckId = self.mLastAckId + 1
	return self.mLastAckId
end

-- start sending join-room requests until acknowledged by server
function cUDPServCon:StartJoinRoom		(sRoomName)
	local iAckId = self:GenerateAckId()
	--~ print("StartJoinRoom",iAckId,sRoomName)
	local f = function () 
		--~ print("StartJoinRoom-TaskStep",self,iAckId,sRoomName) 
		self:SendJoinRoom(iAckId,sRoomName) 
	end
	self:AddTask(f)
	self:AddAckHandler(iAckId,function (pPacket)
		local iRoomId = byte4_r_ntoh(pPacket,1+4)
		--~ print("StartJoinRoom->ack",sRoomName,iRoomId)
		gLastResolvedRoomID = iRoomId
		self.mRoomIdByName[sRoomName] = iRoomId
		self.mRoomNameById[iRoomId] = sRoomName
		self:DelTask(f)
		self:on_joinroom(sRoomName,iRoomId)
		end)
	f()
end

-- start sending leave-room requests until acknowledged by server
function cUDPServCon:StartLeaveRoom		(iRoomId)
	local iAckId = self:GenerateAckId()
	local f = function () self:SendLeaveRoom(iAckId,iRoomId) end
	self:AddTask(f)
	self:AddAckHandler(iAckId,function (pPacket)
		local sRoomName = self:GetRoomNameById(iRoomId)
		self:on_leaveroom(sRoomName,iRoomId)
		self.mRoomIdByName[sRoomName] = nil
		self.mRoomNameById[iRoomId] = nil
		self:DelTask(f)
	end)
	f()
end

-- start sending reliable-message requests until acknowledged by server
function cUDPServCon:StartSendReliable	(iTo,iInterval,pData)
	local iAckId = self:GenerateAckId()
	local iNextSend = RobGetTicks()
	local f = function (t) 
		if (t >= iNextSend) then 
			iNextSend = t + iInterval
			self:SendReliable(iAckId,iTo,iInterval,pData)
		end
	end
	self:AddTask(f)
	self:AddAckHandler(iAckId,function (pPacket) self:DelTask(f) end)
	f(iNextSend)
end

-- ***** ***** ***** ***** ***** room utils

function cUDPServCon:GetRoomNameById	(iRoomId)	return self.mRoomNameById[iRoomId] end
function cUDPServCon:GetRoomIdByName	(sRoomName)	return self.mRoomIdByName[sRoomName] end
function cUDPServCon:IsInRoomByName		(sRoomName)	return self:GetRoomIdByName(sRoomName) ~= nil end
function cUDPServCon:IsInRoomById		(iRoomId)	return self:GetRoomNameById(iRoomId) ~= nil end
function cUDPServCon:GetOwnId			()	return self.mOwnID end

-- ***** ***** ***** ***** ***** notifiers, can be overridden

-- override me !
function cUDPServCon:on_message		(pData,iTo)			print("cUDPServCon:on_message : override me !!!",iTo,pData) end
function cUDPServCon:on_joinroom	(sRoomName,iRoomId)	print("cUDPServCon:on_joinroom : override me !!!",sRoomName,iRoomId) end
function cUDPServCon:on_leaveroom	(sRoomName,iRoomId)	print("cUDPServCon:on_leaveroom : override me !!!",sRoomName,iRoomId) end
function cUDPServCon:on_ownid		(iOwnId)			print("cUDPServCon:on_ownid : override me !!!",iOwnId) end
function cUDPServCon:on_ping		() end -- ping from server

-- ***** ***** ***** ***** ***** stepper

local gErrorCount = 0
function cUDPServCon:ReadStep ()
	-- connect
	if (not self.mOwnID) then self:SendHello() end
	local t = RobGetTicks()
	
	-- handle packets
	while true do 
		local p = self.socket:receive()
		if (not p) then break end
		local iMsgId = byte1_r(p)
--		print("msg:",kMsgTypeTxt[iMsgId] or iMsgId)
		if (iMsgId == kMsgTypes.kMsgType_NormalUDP		) then self:on_message(data_rest(p,1+4),byte4_r_ntoh(p,1)) end
		if (iMsgId == kMsgTypes.kMsgType_ReliableUDP	) then
			local iAckId = byte4_r_ntoh(p,1)
			self:SendAck(iAckId)
			-- on_message only once
			if (not self.mKnownReliableUDPAck[iAckId]) then
				self.mKnownReliableUDPAck[iAckId] = t
				self:on_message(data_rest(p,1+4*3),byte4_r_ntoh(p,1+4*1))
			end
		end
		if (iMsgId == kMsgTypes.kMsgType_Ack			) then self:ExecuteAckHandler(byte4_r_ntoh(p,1),p) end
		if (iMsgId == kMsgTypes.kMsgType_Welcome		) then self.mOwnID = byte4_r_ntoh(p,1) self:on_ownid(self.mOwnID) end
		if (iMsgId == kMsgTypes.kMsgType_Ping			) then self:SendPing() self:on_ping() end
	end
	
	-- check if known-UDP ack memory has old entries that can be discarded
	if (t >= self.miNextKnownReliableUDPAckCheck) then
		self.miNextKnownReliableUDPAckCheck = t + kKnownReliableUDPAckCheckInterval
		-- delete old to avoid memleak
		local tDeadT = t - kKnownReliableUDPAckCheckInterval
		for iAckId,tLastSeen in pairs(self.mKnownReliableUDPAck) do if (tLastSeen < tDeadT) then self.mKnownReliableUDPAck[iAckId] = nil end end
	end
	
	-- step tasks
	for task,v in pairs(self.mTasks) do task(t) end
end
