-- global
require ("lib/MiddleClass")
require ("lib/AnAL")
require ("lib/TSerial")
require ("lib/TEsound")
require ("lib/vector")
require ("lib/utils")
require ("lib/array")
require ("lib/quaternion")
require ("lib/math")
require ("lib/vector2")
gui = require 'lib/Quickie'

--local json = require 'lib.dkjson'
--cron = require 'lib/cron'
--tween = require 'lib/tween'

-- mixed classes
require ("class/gameworld")
require ("class/level")
require ("class/player")
require ("class/generator")
require ("class/item")
require ("class/music")

-- scenes specific
require ("scenes/game")
--require ("scenes/titlemenu")
--require ("scenes/levelmenu")
--require ("scenes/optionmenu")
require ("scenes/intro")
require ("scenes/endscreen")

require ("scenes/menu")
require ("class/circle")
require ("class/hsl")

-- Profile Lua code
_profile_ = false

-- globals
local enable_fps = false
_fps_ = 0
_videomodes_ = nil
_currentmode_ = nil
_screenWidth_ = nil
_screenHeight_ = nil
_globalScale_ = 1
_globalTranslateX_ = 0
_globalTranslateY_ = 0

_ingameFont_ = nil

-- sound yes/no?
_soundmode_ = true

-- current scene
scene = nil

_levels_ = 1

-- global scenes
--_titlemenu_ = nil
--_levelmenu_ = nil
--_optionmenu_ = nil

-- Number of Players
_playercount_ = 1
-- Player key config
_playerconfig_ = {}
_playerconfig_[1] = {}
_playerconfig_[1].key = "q"
_playerconfig_[1].joy = nil
_playerconfig_[2] = {}
_playerconfig_[2].key = "x"
_playerconfig_[2].joy = nil
_playerconfig_[3] = {}
_playerconfig_[3].key = "b"
_playerconfig_[3].joy = nil
_playerconfig_[4] = {}
_playerconfig_[4].key = "m"
_playerconfig_[4].joy = nil

menu_view = {}
menu_view[1] = {
		title="four",
		desc="'four'\nby schattenkind.net",
		{t="Start",cb="ng"},
		{t="Settings",cb="st"},
		{t="Options",cb="op"},
		{t="Credits",cb="cr"},
		{t="Exit",cb="exit"}
	}
menu_view[2] = {
		title="Options",
		desc="Set your options here.",
		{t="Fullscreen",cb="fs"},
		{t="Resolution ("..love.graphics.getWidth().."x"..love.graphics.getHeight()..")",cb="res"},
		{t="Sound (on)",cb="sound"},
		{t="Return",cb="mm"}
	}
menu_view[3] = {
		title="Exit",
		desc="Are you sure you want to quit?",
		{t="Exit",cb="cexit"},
		{t="Return",cb="mm"}
	}
menu_view[4] = {
		title="Credits",
		desc="hagish - Programmer\nmoe - Programmer\ngebirgsbaerbel - Programmer\nSiENcE - Programmer.Music.Art\n",
		{t="Return",cb="mm"}
	}
menu_view[5] = {
		title="Settings",
		desc="Set your keys here.",
		{t="Playercount=".. _playercount_, cb="plcount"},
		{t="Player 1 - " .. " Key: " .. _playerconfig_[1].key, cb="handle1"},
		{t="Player 2 - " .. " Key: " .. _playerconfig_[2].key, cb="handle2"},
		{t="Player 3 - " .. " Key: " .. _playerconfig_[3].key, cb="handle3"},
		{t="Player 4 - " .. " Key: " .. _playerconfig_[4].key, cb="handle4"},
		{t="Return",cb="mm"}
	}

-- before all other: redefining the love.graphics.newImage for setFilter
local __newImage = love.graphics.newImage -- old function
function love.graphics.newImage( ... ) -- new function that sets nearest filter
   local img = __newImage( ... ) -- call old function with all arguments to this function
   img:setFilter( 'nearest', 'nearest' )	-- 'linear', 'linear' or 'nearest', 'nearest'
   return img
end
-- manage debug print outs (disable __print to disable print outs!)
local __print = print --old function
function print( ... )
	__print ( ... )
end

function love.load ()
	_screenWidth_ = love.graphics.getWidth()
	_screenHeight_ = love.graphics.getHeight()
	
	_videomodes_ = love.graphics.getModes()
	_currentmode_ = 1

    local gameIcon = love.graphics.newImage("media/images/icon.png")
    love.graphics.setIcon(gameIcon)

	-- init random
	math.randomseed(os.time())

    font = love.graphics.newImageFont("media/fonts/font1.png",
    "abcdefghijklmnopqrstuvwxyz" ..
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ0" ..
    "123456789 :-!.,\"")
	
	font2 = love.graphics.newFont("media/fonts/Orbitron Black.ttf", 38)

    font3 = love.graphics.newImageFont("media/fonts/font3.png",
    " 0123456789abcdefghijklmnopqrstuvwxyz" ..
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ" ..
    ".-,!:()[]{}<>")

    font4 = love.graphics.newImageFont("media/fonts/font4.png",
    " abcdefghijklmnopqrstuvwxyz" ..
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ0" ..
    "123456789.,!?-+/():;%&`'*#=[]\"")

    _ingameFont_ = font
    _menuFont_ = font

--  	_titlemenu_ = TitleMenu:new()
--  	_levelmenu_ = LevelMenu:new()
--  	_optionmenu_ = OptionMenu:new()

	scene = Menu:new(menu_view)

	if (_profile_) then
		ProFi = require 'ProFi'
		ProFi:start()
	end
end

function love.update(dt)
	if (enable_fps) then
		_fps_ = love.timer.getFPS()
	end

--	cron.update(dt)
--	tween.update(dt)
	
	scene:update(dt)

	-- take in if we have audio
--	love.audio.update()
end

function love.keypressed(key,unicode)
	scene:keypressed(key, unicode)

	-- used for debugging
	if (_profile_ and key == "p") then
		dumpAll()
	end
end

function love.keyreleased(key, unicode)
	scene:keyreleased(key, unicode)
end

function love.quit()
	if (_profile_) then
		ProFi:stop()
		ProFi:writeReport( 'MyProfilingReport.txt' )
	end
	
--	tween.stopAll()
	
	scene:quit()
end

function love.draw()
	scene:draw()
	
	if (enable_fps) then
		love.graphics.print("FPS: " .. _fps_, 1, 1)
	end
end

function getPlayers()
	return _playercount_
end

-- --------------------------------------------------------
-- save state
-- --------------------------------------------------------
--[[
What's wrong with love.filesystem?
Using something like TSerial, it's as simple as :

CODE:
local savedata = {...}
love.filesystem.write('save.sav', TSerial:pack(savedata))
local loaddata = TSerial:unpack(love.filesystem.read('save.sav'))
]]--
-- --------------------------------------------------------
-- used for debugging
-- --------------------------------------------------------
DUMP_IGNORE = {os = 1, io = 1, table = 1, _VERSION = 1, math = 1, love = 1, string = 1, package = 1, _G = 1, DUMP_IGNORE = 1, ndump = 1, xpcall = 1, unpack = 1, type = 1, require = 1, setmetatable = 1, next = 1, pairs = 1, ipairs = 1, dofile = 1, collectgarbage = 1, load = 1, loadfile = 1, loadstring = 1, module = 1, pcall = 1, gcinfo = 1, getmetatable = 1, error = 1, debug = 1, coroutine = 1, assert = 1, tonumber = 1, tostring = 1, setfenv = 1, arg = 1, argv = 1, dumpAll = 1, dumpUserData = 1, getfenv = 1, newproxy = 1, select = 1}

function ndump(object, map, visited, prefix, ignoredMap)
    map = map or {}
    ignoredMap = ignoredMap or {}
    visited = visited or {}
    if object ~= nil and visited[object] == nil then
        visited[object] = true
        for k, v in pairs(object) do
            local useMap = not prefix and DUMP_IGNORE[tostring(k)] ~= nil and ignoredMap or map
            local child = nil
            local vtype = type(v)
            k = tostring(k) 
            if vtype == "table" then
                child = v 
                v = "{}"
            elseif vtype == "string" then
                v = "\"" .. string.gsub(string.gsub(v, "\\", "\\\\"),"\n", "\\n") .. "\"" 
            elseif vtype == "function" then
                v = (prefix and (prefix .. "[\"" .. k .. "\"]") or k) .. " or function() end"
            elseif vtype == "userdata" then
                v = "{} --[[ " .. tostring(v) .. "]]--"
            end
            table.insert(useMap, (prefix and (prefix .. "[\"" .. k .. "\"]") or k) .. " = " .. tostring(v))
            if child then
                ndump(child, useMap, visited, (prefix and (prefix .. "[\"" .. k .. "\"]") or k))
            end
        end
    end
    if not prefix then
        local sf = function(a, b)
            return string.lower(a) < string.lower(b)
        end
        table.sort(map, sf)
        table.sort(ignoredMap, sf)
        return table.concat(map, "\n"), "-- " .. table.concat(ignoredMap, "\n-- ")
    end
end

function dumpAll()
    local r, i = ndump(_G)
    print("--------------------------------------------------")
    print("--[[ Constants, Variables and Functions ]]--")
    print("--------------------------------------------------")
    print(r)
    print("") 
    print("")
    print("-----------------------")
    print("--[[ IGNORED ]]--")
    print("-----------------------")
    print(i)
end
