kYDeathTreshold = love.graphics.getHeight()
kXDeathDelta = 100
kMaxCamTranslateDelta = 5000
kMaxCamScaleDelta = 1
kScreenBorder = 400
kBackgroundGridCount = 5

-- Class
Game = class("Game")

gGame = nil

function Game:initialize(numberOfplayers)
	super.initialize(self)
	print("start loading Game ...")
	
	kYDeathTreshold = love.graphics.getHeight()
	
	self.usebloom = false
	self.useflip = false

	_globalTranslateX_ = 0
	_globalTranslateY_ = 0
	_globalScale_ = 1
	
	self.pitch = 0.8
	self.pitch_var = self.pitch
	
	self.music = Music:new("media/music/music_clean.ogg", self.pitch_var)
	
	local d = 150
	local parts = 100
	
	self.level = Level:new()
	
	self.generator = Generator:new(d, parts, self.level.LevelYBase, 100, 170, 0.1, 0.5, 0.6, 80, 20)
	self.generator.holePercentage = 0.1
	self.generator.obstaclePercentage = 0.25
	self.generator.obstacleUpPercentage = 0.6
	self.generator.obstacleUpHeight = 150
	self.generator.minObstacleSize = 50
	self.generator.maxObstacleSize = 100
	self.generator.obstacleBase = 10
	self.generator.minPlatoHeightDelta = 80
	self.generator.maxPlatoHeightDelta = 120
	self.generator.platoChangePercentage = 0.2
	self.generator.itemsStartAtPart = 20
	
	self.generator:generateAndAddToLevel(self.level)

	self.levelEndX = d * parts
	
	print("LEVEL END", self.levelEndX)

	self.winner = nil
	self.players = {}
	print("numberOfplayers " ..numberOfplayers)
	for id = 1, numberOfplayers do
		table.insert(self.players, Player:new(100, 10, id))
	end

	self.gameWorld = GameWorld:new(self.level, self.players)
	
	print("Game loaded!")
	
	gGame = self
end

------------------------------------------------------------------------
-- Main Functions
------------------------------------------------------------------------

function Game:adjustCamera(dt)
	-- player positions
	local minX = nil
	local maxX = nil
	
	for k,player in pairs(self.players) do
		if not maxX or player.x > maxX then maxX = player.x end
		if not minX or player.x < minX then minX = player.x end
	end
	
	local border = kScreenBorder
	
	-- calculate scaling
	local s = love.graphics.getWidth() / (maxX - minX + 2 * border)
	_globalScale_ = math.min(1, MoveFloatTowards(_globalScale_, s, kMaxCamScaleDelta))
	
	-- position camera that players are visible
	_globalTranslateX_ = MoveFloatTowards(_globalTranslateX_, (-minX + border) * _globalScale_, kMaxCamTranslateDelta )	
	_globalTranslateY_ = MoveFloatTowards(_globalTranslateY_, (self.level.LevelYBase - love.graphics.getHeight() * _globalScale_), kMaxCamTranslateDelta )

--	print(_globalScale_, "->", _globalTranslateX_, _globalTranslateY_)
	
	-- arrow key vector
	local dx = 0
	local dy = 0
	
	if love.keyboard.isDown( "up" ) then dy = -1 end
	if love.keyboard.isDown( "down" ) then dy = 1 end
	if love.keyboard.isDown( "left" ) then dx = -1 end
	if love.keyboard.isDown( "right" ) then dx = 1 end
	
	--~ local f = 500
	--~ _globalTranslateX_ = _globalTranslateX_ - dx * dt * f
	--~ _globalTranslateY_ = _globalTranslateY_ + dy * dt * f
	

end

function Game:update(dt)
	-- if no player anymore -- show endscreen
	if countarr(self.players) == 0 or self.winner then 
		self:quit() 
		scene = Endscreen:new(self.winner) 
	end

	for i=1,5 do self.gameWorld:update(dt/5) end
	
	self.level:update(dt)
	if self.flip and self.useflip then self.useflip = self.flip:update(dt) end

	local maxSpeed = 0

	for k, player in pairs(self.players) do
		maxSpeed = math.max(maxSpeed, player:getSpeed()) 
		player:update(dt) 
	end

	if countarr(self.players) ~= 0 then
		self:adjustCamera(dt)
	end
	
	-- pitch music
	local speedPitchLimit = 400
	if maxSpeed > speedPitchLimit then
		self.pitch_var = MapFromTo(maxSpeed, speedPitchLimit, 1500, self.pitch, 1.5)
	end
	
	self.music:setPitch(MoveFloatTowards(self.music:getPitch(), self.pitch_var, 0.1 * dt))
end

function Game:activateFlip()
	self.useflip = true
	
	-- load flip
	if not self.flip then
		require("effects/flip")
		self.flip = CreateFlipEffect(love.graphics.getWidth(), love.graphics.getHeight())
	end
	
	if self.flip and self.useflip then
		self.flip:trigger() 
		TEsound.play("media/sounds/flip.ogg", "flip")
	end
end

function Game:keypressed(key,unicode)
	-- handle exit
	if key == 'escape' then
		self:quit()
		
--		os.exit() -- TODO

		scene = Menu:new(menu_view)
		return
	end
	
	if key == 'f1' then
		self.usebloom = not self.usebloom
		
		if not self.bloom then
			-- load bloom
			require("effects/bloom")
			self.bloom = CreateBloomEffect(love.graphics.getWidth(), love.graphics.getHeight())
			if self.bloom then
				self.bloom:setThreshold(0.25)
			end
		end
	end
--	if key == 'f2' then
--		self:activateFlip()
--	end
	
	-- player key pressed handling
	for k,player in pairs(self.players) do
		if _playerconfig_[k] and key == _playerconfig_[k].key then
			player.playerKeyPressed = true
		end
	end
end

function Game:keyreleased(key,unicode)
	-- player key pressed handling
	for k,player in pairs(self.players) do
		if _playerconfig_[k] then
		
			player.playerKey = _playerconfig_[k].key
		
			if key == _playerconfig_[k].key then
				player.playerKeyPressed = false
			end
		end
	end
end

function Game:quit()
	self.music:stop()

	-- reset colors
	love.graphics.setColor( 255, 255, 255, 255 )
	
	-- delete ingame state
	print("quit ingame")
end

function Game:draw()
	local targetFunc = nil
	if self.flip and self.useflip then targetFunc = function() 
		love.graphics.push()
		self.flip:predraw() 
		end
	end

	love.graphics.push()
	love.graphics.setBlendMode("additive")
	love.graphics.setLineWidth(1.)
	for i = 1, kBackgroundGridCount + 1 do
		love.graphics.setColor(155 + i * 100/kBackgroundGridCount, 50 + i * 100/kBackgroundGridCount, 0, 255)
		love.graphics.line(0, i * _screenHeight_/(kBackgroundGridCount + 1), _screenWidth_, 
			i * _screenHeight_/(kBackgroundGridCount + 1))
		local x = i * _screenWidth_/(kBackgroundGridCount + 1)
		x = x - (_globalTranslateX_/4) % (_screenWidth_/(kBackgroundGridCount + 1))
		if (i-1) < 2*kBackgroundGridCount / 5 then
			love.graphics.setColor(155 + (i-1) * 100/kBackgroundGridCount, 0, 0, 255)
		elseif (i-1) < 3 * kBackgroundGridCount / 5 then
			love.graphics.setColor(155 + (i-1) * 100/kBackgroundGridCount, 155 + (i-1) * 100/kBackgroundGridCount, 155 + (i-1) * 100/kBackgroundGridCount, 255)
		else
			local val = max(155, 200 + (i-1) * 55/kBackgroundGridCount)
			love.graphics.setColor(val, val, 0, 255)
		end
		love.graphics.line(x, 0, x, _screenHeight_)
	end
	love.graphics.setBlendMode("alpha")

	-- translate after scaling
	love.graphics.translate(_globalTranslateX_, _globalTranslateY_)
	-- scale global
	love.graphics.scale(_globalScale_, _globalScale_)

	-- bloom pre
	if self.usebloom and self.bloom then 
		self.bloom:predraw()
	elseif targetFunc ~= nil then
		targetFunc()
	end
	-----------------
	self.level:draw()

	for k,player in pairs(self.players) do player:draw(false) end

	-----------------
	-- bloom post
	if self.usebloom or not self.useflip then
		love.graphics.pop()
	end
	
	if self.usebloom and self.bloom then 
		self.bloom:postdraw(targetFunc)
	end

	-- late draw
	love.graphics.translate(_globalTranslateX_, _globalTranslateY_)
	love.graphics.scale(_globalScale_, _globalScale_)
	if not self.usebloom and self.useflip then
		love.graphics.pop()
	end
	
	if self.flip and self.useflip then 
		love.graphics.pop()		
		targetFunc = self.flip:postdraw() 
	end

	--------------------------
	-- RESET global scale
	--------------------------
	love.graphics.scale(1, 1)
end
